@@ -116,26 +116,34 @@ int main(int argc, char *argv[])
 	int len;
 	int drive, i, totaltime;
 	long int cksum=0;
+	int musicbrainz=0;
 	unsigned char first=1, last=1;
 	char *devicename=DEVICE_NAME;
 	struct cdrom_tochdr hdr;
 	struct cdrom_tocentry *TocEntry;
+	char *command=argv[0];
 #if defined(__OpenBSD__) || defined(__NetBSD__)
 	struct ioc_read_toc_entry t;
 #elif defined(__APPLE__)
 	dk_cd_read_disc_info_t discInfoParams;
 #endif
 
+	if (argc >= 2 && ! strcmp(argv[1], "--musicbrainz")) {
+		musicbrainz = 1;
+		argc--;
+		argv++;
+	}
 	if (argc == 2) {
 		devicename = argv[1];
 	} else if (argc > 2) {
-		fprintf(stderr, "Usage: %s [devicename]\n", argv[0]);
+		fprintf(stderr, "Usage: %s [--musicbrainz] [devicename]\n",
+			command);
 		exit(1);
 	}
 
 	drive = open(devicename, O_RDONLY | O_NONBLOCK);
 	if (drive < 0) {
-		fprintf(stderr, "%s: %s: ", argv[0], devicename);
+		fprintf(stderr, "%s: %s: ", command, devicename);
 		perror("open");
 		exit(1);
 	}
@@ -146,13 +154,13 @@ int main(int argc, char *argv[])
 	discInfoParams.bufferLength = sizeof(hdr);
 	if (ioctl(drive, DKIOCCDREADDISCINFO, &discInfoParams) < 0
 		|| discInfoParams.bufferLength != sizeof(hdr)) {
-		fprintf(stderr, "%s: %s: ", argv[0], devicename);
+		fprintf(stderr, "%s: %s: ", command, devicename);
 		perror("DKIOCCDREADDISCINFO");
 		exit(1);
 	}
 #else
 	if (ioctl(drive, CDROMREADTOCHDR, &hdr) < 0) {
-		fprintf(stderr, "%s: %s: ", argv[0], devicename);
+		fprintf(stderr, "%s: %s: ", command, devicename);
 		perror("CDROMREADTOCHDR");
 		exit(1);
 	}
@@ -167,7 +175,7 @@ int main(int argc, char *argv[])
 	if (!TocEntry) {
 		fprintf(stderr,
 			"%s: %s: Can't allocate memory for TOC entries\n",
-			argv[0], devicename);
+			command, devicename);
 		exit(1);
 	}
 #if defined(__OpenBSD__) 
@@ -182,7 +190,7 @@ int main(int argc, char *argv[])
 	memset(TocEntry, 0, len);
 	
 	if (ioctl(drive, CDIOREADTOCENTRYS, (char *) &t) < 0) {
-		fprintf(stderr, "%s: %s: ", argv[0], devicename);
+		fprintf(stderr, "%s: %s: ", command, devicename);
 		perror("CDIOREADTOCENTRYS");
 	}
 #elif defined(__APPLE__)
@@ -196,7 +204,7 @@ int main(int argc, char *argv[])
 		trackInfoParams.buffer = &TocEntry[i];
 
 		if (ioctl(drive, DKIOCCDREADTRACKINFO, &trackInfoParams) < 0) {
-			fprintf(stderr, "%s: %s: ", argv[0], devicename);
+			fprintf(stderr, "%s: %s: ", command, devicename);
 			perror("DKIOCCDREADTRACKINFO");
 		}
 	}
@@ -212,7 +220,7 @@ int main(int argc, char *argv[])
 		TocEntry[i].cdte_track = i + 1;
 		TocEntry[i].cdte_format = CDROM_LBA;
 		if (ioctl(drive, CDROMREADTOCENTRY, &TocEntry[i]) < 0) {
-			fprintf(stderr, "%s: %s: ", argv[0], devicename);
+			fprintf(stderr, "%s: %s: ", command, devicename);
 			perror("CDROMREADTOCENTRY");
 		}
 	}
@@ -220,7 +228,7 @@ int main(int argc, char *argv[])
 	TocEntry[last].cdte_track = CDROM_LEADOUT;
 	TocEntry[last].cdte_format = CDROM_LBA;
 	if (ioctl(drive, CDROMREADTOCENTRY, &TocEntry[i]) < 0) {
-		fprintf(stderr, "%s: %s: ", argv[0], devicename);
+		fprintf(stderr, "%s: %s: ", command, devicename);
 		perror("CDROMREADTOCENTRY");
 	}
 #endif
@@ -240,20 +248,26 @@ int main(int argc, char *argv[])
 		    ((TocEntry[0].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);
 
 	/* print discid */
-	printf("%08lx", (cksum % 0xff) << 24 | totaltime << 8 | last);
+	if (! musicbrainz)
+		printf("%08lx ", (cksum % 0xff) << 24 | totaltime << 8 | last);
 
 	/* print number of tracks */
-	printf(" %d", last);
+	printf("%d", last);
 
 	/* print frame offsets of all tracks */
 	for (i = 0; i < last; i++) {
 		printf(" %d", TocEntry[i].cdte_track_address + CD_MSF_OFFSET);
 	}
-	
-	/* print length of disc in seconds */
-        printf(" %d\n", (TocEntry[last].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);
+
+	if (musicbrainz) {
+		printf(" %d\n", TocEntry[last].cdte_track_address + CD_MSF_OFFSET);
+	} else {
+		/* print length of disc in seconds */
+		printf(" %d\n", (TocEntry[last].cdte_track_address + CD_MSF_OFFSET) / CD_FRAMES);
+	}
 
         free(TocEntry);
+	close(drive);
 
         return 0;
 }
